@@ -19,7 +19,7 @@
  *
  * Author: Bert Bos
  * Created: 2 Dec 1998
- * Version: $Id: hxincl.c,v 1.5 2009/03/25 18:46:04 bbos Exp $
+ * Version: $Id: hxincl.c,v 1.6 2010/04/26 13:14:21 bbos Exp $
  *
  **/
 #include <config.h>
@@ -198,7 +198,7 @@ void handle_comment(void *clientdata, string commenttext)
     }
 
     /* If we have a substitution for it, use that instead */
-    if ((s = dict_find(substitutions, url))) url = s;
+    if (substitutions && (s = dict_find(substitutions, url))) url = s;
 
     /* Get the file and recursively parse it */
     s = URL_s_absolutize(base, url);
