@@ -37,15 +37,17 @@
  *
  * Author: Bert Bos <bert@w3.org>
  * Created: 11 Apr 2000
- * Version: $Id: hxindex.c,v 1.6 2009/03/25 18:46:19 bbos Exp $
+ * Version: $Id: hxindex.c,v 1.9 2010/01/15 00:23:24 bbos Exp $
  *
  **/
 #include <config.h>
 #include <assert.h>
+#include <locale.h>
+#include <wchar.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <time.h>
+#include <iconv.h>
 #include <unistd.h>
 #if STDC_HEADERS
 # include <string.h>
@@ -102,9 +104,12 @@ int strerror_r(int errnum, char *buf, size_t n);
 #define NO_NUM "no-num"		/* Class of elements without a section # */
 
 typedef struct _indexterm {
-  string term, url;
+  string url;
   int importance;		/* 1 (low) or 2 (high) */
   string secno;
+  string *terms;		/* Array of subterms */
+  wchar_t **sortkeys;		/* Array of normalized subterms */
+  int nrkeys;			/* Length of term and sortkeys arrays */
 } *Indexterm;
 
 static Tree tree;
@@ -113,7 +118,7 @@ static string base = NULL;	/* (Rel.) URL of output file */
 static string indexdb = NULL;	/* Persistent store of terms */
 static string* userclassnames = NULL;	/* Persistent store of class names */
 static FILE *globalfile;	/* Must be global for twalk */
-static string globalprevious;	/* Must be global for twalk */
+static Indexterm globalprevious; /* Must be global for twalk */
 static string globalurlprevious;/* Must be global for twalk */
 static Boolean bctarget = True;	/* Add <A name=> after IDs */
 static Boolean use_secno = False; /* Anchor text is "#" instead of section # */
@@ -192,48 +197,6 @@ static void handle_endtag(void *clientdata, string name)
   tree = html_pop(tree, name);
 }
 
-/* next -- return pointer to next non-punctuation, non-markup char, or NULL */
-static conststring next(conststring s, const char stopchar)
-{
-  Boolean in_ent = False, in_dquote = False, in_squote = False, in_tag = False;
-
-  for (;; s++) {
-    if (*s == '\0' || *s == stopchar) return NULL;
-    else if (in_ent) in_ent = *s != ';';
-    else if (in_dquote) in_dquote = *s != '"';
-    else if (in_squote) in_squote = *s != '\'';
-    else if (in_tag) {
-      if (*s == '"') in_dquote = True;
-      else if (*s == '\'') in_squote = True;
-      else in_tag = *s != '>';
-    } else {
-      if (*s == '&') in_ent = True;
-      else if (*s == '<') in_tag = True;
-      else if (isalnum(*s)) return s;
-    }
-  }
-}
-
-/* folding_cmp -- compare two string, ignoring case and punctuation */
-static int folding_cmp(conststring a, conststring b, const char stopchar)
-{
-  /* Stopchar determines if we compare full strings or subterms up to '\1' */
-  assert(stopchar == '\0' || stopchar == '\1');
-  assert(a);
-  assert(b);
-
-  for (;;) {
-    a = next(a, stopchar);
-    b = next(b, stopchar);
-    if (!a) return b ? -1 : 0;
-    else if (!b) return 1;
-    else if (tolower(*a) < tolower(*b)) return -1;
-    else if (tolower(*a) > tolower(*b)) return 1;
-    else a++, b++;
-  }
-  assert(! "Cannot happen!");
-}
-
 /* trim -- remove leading and trailing white space */
 static void trim(string s)
 {
@@ -245,6 +208,103 @@ static void trim(string s)
   s[j-i] = '\0';
 }
 
+/* parse_subterms -- parse s to create terms & sortkeys array in an Indexterm */
+static void parse_subterms(const Indexterm term, const conststring s)
+{
+  enum {TEXT, TAG, DQUOTE, SQUOTE} state;
+  string h, k, p, q;
+  iconv_t cd;
+  size_t len, len2;
+  int i;
+
+  /* Create the terms array and count the number of subterms */
+  h = newstring(s);
+  term->nrkeys = 1;
+  newarray(term->terms, 1);
+  term->terms[0] = h;
+  while ((k = strstr(h, "!!")) || (k = strstr(h, "::"))) {
+    h = k + 2;
+    *k = '\0';
+    renewarray(term->terms, term->nrkeys + 1);
+    trim(h);				/* Remove leading & trailing space */
+    term->terms[term->nrkeys] = h;	/* All terms point into h */
+    term->nrkeys++;
+  }
+
+  /* Create the sortkeys array by normalizing each term */
+  newarray(term->sortkeys, term->nrkeys);
+  for (i = 0; i < term->nrkeys; i++) {
+
+    /* First remove mark-up and expand the standard XML entities */
+    h = newstring(term->terms[i]);
+    for (state = TEXT, p = q = h; *p; p++) {
+      switch (state) {
+      case TEXT:
+	if (*p == '<') state = TAG;
+	else if (hasprefix(p, "&lt;"))   {*(q++) = '<'; p += 3;}
+	else if (hasprefix(p, "&#60;"))  {*(q++) = '<'; p += 4;}
+	else if (hasprefix(p, "&#x3c;")) {*(q++) = '<'; p += 5;}
+	else if (hasprefix(p, "&#x3C;")) {*(q++) = '<'; p += 5;}
+	else if (hasprefix(p, "&gt;"))   {*(q++) = '>'; p += 3;}
+	else if (hasprefix(p, "&#62;"))  {*(q++) = '>'; p += 4;}
+	else if (hasprefix(p, "&#x3e;")) {*(q++) = '>'; p += 5;}
+	else if (hasprefix(p, "&#x3E;")) {*(q++) = '>'; p += 5;}
+	else if (hasprefix(p, "&quot;")) {*(q++) = '"'; p += 5;}
+	else if (hasprefix(p, "&#34;"))  {*(q++) = '"'; p += 4;}
+	else if (hasprefix(p, "&#x22;")) {*(q++) = '"'; p += 5;}
+	else if (hasprefix(p, "&amp;"))  {*(q++) = '&'; p += 4;}
+	else if (hasprefix(p, "&#38;"))  {*(q++) = '&'; p += 4;}
+	else if (hasprefix(p, "&#x26;")) {*(q++) = '&'; p += 5;}
+	else *(q++) = tolower(*p);
+	break;
+      case TAG:
+	if (*p == '>') state = TEXT;
+	else if (*p == '"') state = DQUOTE;
+	else if (*p == '\'') state = SQUOTE;
+	break;
+      case DQUOTE:
+	if (*p == '"') state = TAG;
+	break;
+      case SQUOTE:
+	if (*p == '\'') state = TAG;
+	break;
+      default:
+	assert(!"Cannot happen!");
+      }
+    }
+    *q = '\0';
+
+    /* Then convert from UTF-8 to wchar_t */
+    cd = iconv_open("wchar_t", "UTF-8");
+    if (cd == (iconv_t)(-1)) {perror("hxindex"); exit(1);}
+    len = strlen(h) + 1;
+    newarray(term->sortkeys[i], len); /* Large enough */
+    p = (string) term->sortkeys[i];
+    len2 = len * sizeof(term->sortkeys[i][0]);
+    if (iconv(cd, &h, &len, &p, &len2) == (size_t)(-1)) {
+      perror("hxindex"); exit(1);
+    }
+    /* *p = L'\0'; */
+    if (iconv_close(cd) == -1) {perror("hxindex"); exit(1);}
+  }
+}
+
+/* folding_cmp -- compare two arrays of sort keys */
+static int folding_cmp(wchar_t **a, const int alen, wchar_t **b, const int blen)
+{
+  int i, j;
+
+  assert(a && alen >= 0);
+  assert(b && blen >= 0);
+  for (i = 0;; i++) {
+    if (i == alen) return i == blen ? 0 : -1;
+    if (i == blen) return 1;
+    assert(a[i] && b[i]);
+    if ((j = wcscoll(a[i], b[i])) != 0) return j;
+  }
+  assert(! "Cannot happen!");
+}
+
 /* indent -- print newline and n times 2 spaces */
 static void indent(int n)
 {
@@ -253,34 +313,35 @@ static void indent(int n)
 }
 
 /* print_full_term -- print the full term for use in a TITLE attribute */
-static void print_full_term(const string term)
+static void print_full_term(const Indexterm term)
 {
-  int j;
-  typedef enum {IN_CONTENT, IN_TAG, IN_DSTRING, IN_SSTRING} Tag_state;
-  Tag_state state;
+  enum {TEXT, TAG, DQUOTE, SQUOTE} state;
+  string h;
+  int i;
 
-  for (j = 0, state = IN_CONTENT; term[j]; j++) {
-    switch (state) {
-    case IN_CONTENT:
-      if (term[j] == '\1') putchar(' ');
-      else if (term[j] == '"') fputs("&quot;", stdout);
-      else if (term[j] == '<') state = IN_TAG;
-      else putchar(term[j]);
-      break;
-    case IN_TAG:
-      if (term[j] == '>') state = IN_CONTENT;
-      else if (term[j] == '"') state = IN_DSTRING;
-      else if (term[j] == '\'') state = IN_SSTRING;
-      break;
-    case IN_DSTRING:
-      if (term[j] == '"') state = IN_TAG;
-      break;
-    case IN_SSTRING:
-      if (term[j] == '\'') state = IN_TAG;
-      break;
-    default:
-      assert(! "Cannot happen\n");
-    }
+  for (state = TEXT, i = 0; i < term->nrkeys; i++) {
+    if (i != 0) fputs(", ", stdout);
+    for (h = term->terms[i]; *h; h++)
+      switch (state) {
+      case TEXT:
+	if (*h == '<') state = TAG;
+	else if (*h == '"') fputs("&quot;", stdout);
+	else putchar(*h);
+	break;
+      case TAG:
+	if (*h == '>') state = TEXT;
+	else if (*h == '"') state = DQUOTE;
+	else if (*h == '\'') state = SQUOTE;
+	break;
+      case DQUOTE:
+	if (*h == '"') state = TAG;
+	break;
+      case SQUOTE:
+	if (*h == '\'') state = TAG;
+	break;
+      default:
+	assert(!"Cannot happen!");
+      }
   }
 }
 
@@ -288,44 +349,43 @@ static void print_full_term(const string term)
 static void write_index_item(const void *term1, const VISIT which,
 			     const int depth)
 {
-  string sub[MAXSUBS], oldsub[MAXSUBS];
-  string p;
   Indexterm term = *(Indexterm*)term1;
-  int i, j, n, oldn;
+  int i, j;
 
   if (which != postorder && which != leaf) return;
 
-  sub[0] = term->term;
-  n = 1;
-  for (p = strstr(term->term, "\1\1"); p; p = strstr(p + 2, "\1\1"))
-    sub[n++] = p + 2;
-  sub[n] = sub[n-1] + strlen(sub[n-1]) + 2;
-
-  assert(globalprevious);
-
-  oldsub[0] = globalprevious;
-  oldn = 1;
-  for (p = strstr(globalprevious, "\1\1"); p; p = strstr(p + 2, "\1\1"))
-    oldsub[oldn++] = p + 2;
-  oldsub[oldn] = oldsub[oldn-1] + strlen(oldsub[oldn-1]) + 2;
-
   /* Count how many subterms are equal to the previous entry */
-  for (i = 0; i < min(n, oldn)
-	 && folding_cmp(sub[i], oldsub[i], '\1') == 0; i++);
+  i = 0;
+  while (i < min(term->nrkeys, globalprevious->nrkeys) &&
+	 !folding_cmp(term->sortkeys + i, 1, globalprevious->sortkeys + i, 1))
+    i++;
 
-  for (j = oldn - 1; j > i; j--) {indent(j); printf("</ul>");}
-  if (n > oldn && oldn == i) {indent(i); printf("<ul>");}
+  /* Close lists as needed */
+  for (j = globalprevious->nrkeys - 1; j > i; j--) {
+    indent(j);
+    printf("</ul>");
+  }
 
-  /* Print new (sub)terms */
-  for (j = i; j < n; j++) {
-    indent(j); printf("<li>");
-    for (p = sub[j]; p != sub[j+1] - 2; p++) putchar(*p);
-    if (j != n - 1) {indent(j+1); printf("<ul>");}
+  /* Open a list if needed */
+  if (term->nrkeys > globalprevious->nrkeys && globalprevious->nrkeys == i) {
+    indent(i);
+    printf("<ul>");
+  }
+
+  /* Print new subterms, if any */
+  for (j = i; j < term->nrkeys; j++) {
+    indent(j);
+    printf("<li>%s", term->terms[j]);
+    if (j != term->nrkeys - 1) {
+      indent(j + 1);
+      printf("<ul>");
+    }
   }
 
   int listmode = 0;
 
-  if (folding_cmp(globalprevious, term->term, '\0') == 0)
+  if (folding_cmp(globalprevious->sortkeys, globalprevious->nrkeys,
+		  term->sortkeys, term->nrkeys) == 0)
     if (globalurlprevious) {
       string d = strchr(globalurlprevious,'#');
       if (d)
@@ -338,12 +398,12 @@ static void write_index_item(const void *term1, const VISIT which,
   switch (term->importance) {
     case 1:
       printf("%s <a href=\"%s\" title=\"", listmode ? ";" : ",", term->url);
-      print_full_term(term->term);
+      print_full_term(term);
       printf("\">%s</a>", use_secno ? term->secno : "#");
       break;
     case 2:
       printf("%s <a href=\"%s\" title=\"", listmode ? ";" : ",", term->url);
-      print_full_term(term->term);
+      print_full_term(term);
       printf("\"><strong>%s</strong></a>", use_secno ? term->secno : "#");
       break;
     default:
@@ -351,27 +411,29 @@ static void write_index_item(const void *term1, const VISIT which,
   }
 
   /* Remember this term */
-  globalprevious = term->term;
+  globalprevious = term;
   globalurlprevious = term->url;
+
 }
 
 /* mkindex -- write out an index */
 static void mkindex(Indexterm terms)
 {
-  string p, h;
+  int i;
 
   printf("<ul class=\"indexlist\">");
-  globalprevious = "zzzzzzzzzzzzzzzzzz"; /* An unlikely alphanumeric string */
+
+  /* Initialize globalprevious to a term with an unlikely sortkey */
+  new(globalprevious);
+  globalprevious->nrkeys = 1;
+  newarray(globalprevious->sortkeys, globalprevious->nrkeys);
+  newarray(globalprevious->sortkeys[0], 15);
+  wcscpy(globalprevious->sortkeys[0], L"zzzzzzzzzzzzzz");
+
   twalk(terms, write_index_item);
 
   /* Close all open lists */
-  p = globalprevious;
-  while (p) {
-    printf("\n</ul>");
-    h = p;
-    p = strstr(h, "\1\1");
-    if (p) p += 2;
-  }
+  for (i = 0; i < globalprevious->nrkeys; i++) printf("\n</ul>");
 }
 
 /* expand -- write the tree, add <A NAME> if needed and replace <!--index--> */
@@ -446,18 +508,18 @@ static void expand(Tree t, Boolean *write, Indexterm terms)
 static int termcmp(const void *a1, const void *b1)
 {
   Indexterm a = (Indexterm)a1, b = (Indexterm)b1;
+  int r;
 
   assert(a);
   assert(b);
-  assert(a->term);
-  assert(b->term);
+  assert(a->sortkeys);
+  assert(b->sortkeys);
+  assert(a->nrkeys > 0);
+  assert(b->nrkeys > 0);
 
-  switch (folding_cmp(a->term, b->term, '\0')) {
-  case -1: return -1;
-  case 0: return strcmp(a->url, b->url); /* Terms are equal, use URL instead */
-  case 1: return 1;
-  default: assert(! "Cannot happen!");
-  }
+  r = folding_cmp(a->sortkeys, a->nrkeys, b->sortkeys, b->nrkeys);
+  if (r != 0) return r;
+  return strcmp(a->url, b->url); /* Terms are equal, compare URL instead */
 }
 
 /* copy_contents -- recursively expand contents of element t into a string */
@@ -514,23 +576,19 @@ static void copy_contents(Tree t, string *s)
 static void copy_to_index(Tree t, Indexterm *terms, int importance,
 			  conststring secno)
 {
-  conststring id, title0;
-  string title, h;
+  conststring id, title;
+  string h;
   Indexterm term;
   int i, n;
 
   id = get_attrib(t, "id");
 #ifdef USE_DATA_ATTRIBUTE
-  if (! (title0 = get_attrib(t, "data-index")))
+  if (! (title = get_attrib(t, "data-index")))
 #endif
-    title0 = get_attrib(t, "title");
+    title = get_attrib(t, "title");
 
   /* Get term either from title attribute or contents */
-  if (title0) {
-    /* Hack: replace !! and :: by \1\1, so that the string sorts better */
-    title = newstring(title0);
-    for (h = strstr(title, "!!"); h; h = strstr(h + 2, "!!")) h[0] = h[1] = '\1';
-    for (h = strstr(title, "::"); h; h = strstr(h + 2, "::")) h[0] = h[1] = '\1';
+  if (title) {
 
     i = 0;
     while (title[i]) {
@@ -540,9 +598,10 @@ static void copy_to_index(Tree t, Indexterm *terms, int importance,
       term->secno = newstring(secno);
       term->url = NULL;
       strapp(&term->url, base, "#", id, NULL);
-      term->term = newnstring(title + i, n);
+      h = newnstring(title + i, n);
+      parse_subterms(term, h);
       if (! tsearch(term, (void**)terms, termcmp))
-	errexit("Out of memory while parsing term %s\n", term->term);
+	errexit("Out of memory while parsing term %s\n", h);
       i += n;
       if (title[i]) i++;			/* Skip '|' */
     }
@@ -554,15 +613,18 @@ static void copy_to_index(Tree t, Indexterm *terms, int importance,
 
   } else {					/* Recursively copy contents */
 
-    new(term);
-    term->importance = importance;
-    term->secno = newstring(secno);
-    term->url = term->term = NULL;
-    strapp(&term->url, base, "#", id, NULL);
-    copy_contents(t, &term->term);
-    if (term->term)				/* Non-empty contents */
+    h = NULL;
+    copy_contents(t, &h);
+    if (h) {					/* Non-empty contents */
+      new(term);
+      term->importance = importance;
+      term->secno = newstring(secno);
+      term->url = NULL;
+      strapp(&term->url, base, "#", id, NULL);
+      parse_subterms(term, h);
       if (! tsearch(term, (void**)terms, termcmp))
-	errexit("Out of memory while parsing term %s\n", term->term);
+	errexit("Out of memory while parsing term %s\n", h);
+    }
 
   }
 }
@@ -613,6 +675,7 @@ static void load_index(const string indexdb, Indexterm *terms)
   int n, n3;
   char line[MAXSTR];
   Indexterm term;
+  string h;
 
   if (! (f = fopen(indexdb, "r"))) return;	/* Assume file not found... */
 
@@ -621,7 +684,7 @@ static void load_index(const string indexdb, Indexterm *terms)
     n = strcspn(line, "\t");
     if (line[n] != '\t') errexit("Illegal syntax in %s\n", indexdb);
     new(term);
-    term->term = newnstring(line, n);
+    h = newnstring(line, n);
     switch (line[n + 1]) {
       case '1': term->importance = 1; break;
       case '2': term->importance = 2; break;
@@ -632,6 +695,7 @@ static void load_index(const string indexdb, Indexterm *terms)
     if (line[n3] != '\t') errexit("Illegal syntax in %s\n", indexdb);
     term->url = newnstring(line + n + 3, n3 - n - 3);
     term->secno = newstring(line + n3 + 1);
+    parse_subterms(term, h);
     if (! tsearch(term, (void**)terms, termcmp))
       errexit("Out of memory while loading %s\n", indexdb);
   }
@@ -643,10 +707,16 @@ static void load_index(const string indexdb, Indexterm *terms)
 static void save_a_term(const void *term1, const VISIT which, const int dp)
 {
   Indexterm term = *(Indexterm*)term1;
+  int i;
 
-  if (which == endorder || which == leaf)
-    fprintf(globalfile, "%s\t%d\t%s\t%s\n", term->term, term->importance,
-	    term->url, term->secno);
+  if (which == endorder || which == leaf) {
+    for (i = 0; i < term->nrkeys; i++) {
+      if (i > 0) fprintf(globalfile, "!!");
+      fprintf(globalfile, "%s", term->terms[i]);
+    }
+    fprintf(globalfile, "\t%d\t%s\t%s\n", term->importance, term->url,
+	    term->secno);
+  }
 }
 
 /* save_index -- write terms to file */
@@ -724,6 +794,9 @@ int main(int argc, char *argv[])
 
   if (!base) base = newstring("");
 
+  /* Apply user's locale */
+  setlocale(LC_ALL, "");
+
   /* Read the index DB into memory */
   if (indexdb) load_index(indexdb, &termtree);
 
