@@ -6,7 +6,7 @@
  *
  * Created 9 May 1998
  * Bert Bos <bert@w3.org>
- * $Id: hxnormalize.c,v 1.4 2009/03/08 15:24:00 bbos Exp $
+ * $Id: hxnormalize.c,v 1.5 2009/10/06 20:03:15 bbos Exp $
  */
 #include <config.h>
 #include <stdio.h>
@@ -35,6 +35,7 @@ static Boolean do_xml = False;
 static Boolean do_endtag = False;
 static Boolean has_errors = False;
 static Boolean do_doctype = True;
+static Boolean clean_span = False;
 
 
 /* handle_error -- called when a parse error occurred */
@@ -195,6 +196,12 @@ static void pp(Tree n, Boolean preformatted, Boolean allow_text)
       else outln(">", False);
       break;
     case Element:
+      if (clean_span && eq(n->name, "span") && ! n->attribs) {
+	/* Omit start and end tags, print just the children. */
+	for (l = n->children; l != NULL; l = l->sister)
+	  pp(l, preformatted, True);
+	break;
+      }
       if (!preformatted && break_before(n->name)) outln(NULL, False);
       out("<", preformatted); out(n->name, preformatted);
       if (break_before(n->name)) inc_indent();
@@ -275,13 +282,14 @@ int main(int argc, char *argv[])
   set_emptytag_handler(handle_emptytag);
   set_endtag_handler(handle_endtag);
 
-  while ((c = getopt(argc, argv, "edxi:l:")) != -1)
+  while ((c = getopt(argc, argv, "edxi:l:s")) != -1)
     switch (c) {
     case 'e': do_endtag = True; break;
     case 'x': do_xml = True; break;
     case 'd': do_doctype = False; break;
     case 'i': set_indent(atoi(optarg)); break;
     case 'l': set_linelen(atoi(optarg)); break;
+    case 's': clean_span = True; break;
     default: usage(argv[0]);
     }
   if (optind == argc) yyin = stdin;
