@@ -22,7 +22,7 @@
  *
  * Author: Bert Bos <bert@w3.org>
  * Created: 4 August 2000
- * Version: $Id: hxref.c,v 1.2 2009/01/19 19:22:03 bbos Exp $
+ * Version: $Id: hxref.c,v 1.4 2010/04/27 14:30:54 bbos Exp $
  **/
 
 #include <config.h>
@@ -267,11 +267,15 @@ static void find_instances(Tree tree)
   conststring title;
   Node *h;
 
+  if (!tree) return;
+
   switch (tree->tp) {
     case Text: case Comment: case Declaration: case Procins:
+      find_instances(tree->sister);
       break;
     case Root:
-      for (h = tree->children; h; h = h->sister) find_instances(h);
+      find_instances(tree->children);		/* Recurse over children */
+      find_instances(tree->sister);		/* Recurse over siblings */
       break;
     case Element:
       if (eq(tree->name, "a") || eq(tree->name, "dfn"))
@@ -293,18 +297,19 @@ static void find_instances(Tree tree)
 	else					/* Get flattened contents */
 	  entry.key = normalize(get_contents(tree));
 	if (! (e = hsearch(entry, FIND))) {	/* If not an instance */
-	  for (h = tree->children; h; h = h->sister) find_instances(h);
-	} else if (eq(tree->name, "span")) { /* Found an instance */
-	  rename_elt(tree, "a");	     /* Turn the span into an a */
+	  find_instances(tree->children);	/* Recurse over children */
+	} else if (eq(tree->name, "span")) {	/* Found an instance */
+	  rename_elt(tree, "a");		/* Turn the span into an a */
 	  set_attrib(tree, "href", e->data);
 	} else {
-	  h = wrap_contents(tree, "a", NULL);	/* Wrap contents in an <a> */
-	  set_attrib(h, "href", e->data);
+	  tree = wrap_elt(tree, "a", NULL);	/* Wrap element in an <a> */
+	  set_attrib(tree, "href", e->data);
 	}
 	dispose(entry.key);
       } else {					/* Not an inline element */
-        for (h = tree->children; h; h = h->sister) find_instances(h);
+	find_instances(tree->children);		/* Recurse over children */
       }
+      find_instances(tree->sister);		/* Recurse over siblings */
       break;
     default:
       assert(!"Cannot happen");
