@@ -149,8 +149,8 @@ static FILE *open_http2(const URL server, const conststring url,
   /* To do:handle 305 Use Proxy */
 
   /* Return the body of the stream */
-  if (delete_response) dict_delete(response);
-  if (delete_headers) dict_delete(headers);
+  if (response && delete_response) dict_delete(response);
+  if (headers && delete_headers) dict_delete(headers);
   return f;
 }
 
