@@ -20,7 +20,7 @@ EOF
 
 # The echo adds a newline at the end of the file
 #
-(./hxnum $TMP1 | ./hxindex -t -n; echo) >$TMP2
+(./hxnum $TMP1 | LC_ALL=C ./hxindex -t -n; echo) >$TMP2
 
 cat >$TMP3 <<EOF
 <html><body><h1><span class="secno">1. </span>Heading 0</h1><h1 class="no-num">Heading 1</h1><p><dfn id="a-term">A-term</dfn>
@@ -28,8 +28,8 @@ cat >$TMP3 <<EOF
 </p><h3><span class="secno">1.1.1. </span>Heading 3</h3><p><dfn id="z-term">Z-term</dfn>
 </p><h1><span class="secno">2. </span>Index</h1><!--begin-index-->
 <ul class="indexlist">
-<li>A-term, <a href="#a-term" title="A-term"><strong>#</strong></a>
 <li>&lt;M-term&gt;, <a href="#ltm-termgt" title="&lt;M-term&gt;"><strong>1.1.</strong></a>
+<li>A-term, <a href="#a-term" title="A-term"><strong>#</strong></a>
 <li>Z-term, <a href="#z-term" title="Z-term"><strong>1.1.1.</strong></a>
 </ul><!--end-index--></body></html>
 EOF
