@@ -20,7 +20,7 @@ EOF
 
 # The echo adds a newline at the end of the file
 #
-(./hxnum $TMP1 | ./hxindex -t -n -f; echo) >$TMP2
+(./hxnum $TMP1 | LC_ALL=C ./hxindex -t -n -f; echo) >$TMP2
 
 cat >$TMP3 <<EOF
 <html><body><h1><span class="secno">1. </span>Heading 0</h1><h1 class="no-num">Heading 1</h1><p><dfn id="a-term">A-term</dfn>
@@ -29,9 +29,9 @@ cat >$TMP3 <<EOF
 </p><h1><span class="secno">2. </span>Index</h1><ul class="indexlist">
 <li>term
   <ul>
-  <li>A, <a href="#a-term" title="term  A"><strong>#</strong></a>
-  <li>M, <a href="#ltm-termgt" title="term  M"><strong>1.1.</strong></a>
-  <li>Z, <a href="#z-term" title="term  Z"><strong>1.1.1.</strong></a>
+  <li>A, <a href="#a-term" title="term, A"><strong>#</strong></a>
+  <li>M, <a href="#ltm-termgt" title="term, M"><strong>1.1.</strong></a>
+  <li>Z, <a href="#z-term" title="term, Z"><strong>1.1.1.</strong></a>
 </ul>
 </ul></body></html>
 EOF
