@@ -16,6 +16,9 @@ TMP2=`mktemp` || exit 1
 	<a href="../link4">link4</a>
 	<a href="../../link5">link5</a>
 	<img src="img.png" alt="img">
+	<a href="./">link6</a>
+	<a href="./link7">link7</a>
+	<a href="../.././link7">link8</a>
 EOF
 cat >$TMP2 <<-EOF
 	link	rel	http://example.org/link0
@@ -30,6 +33,9 @@ cat >$TMP2 <<-EOF
 	a		http://example.org/link4
 	a		http://example.org/link5
 	img		http://example.org/base/img.png
+	a		http://example.org/base/
+	a		http://example.org/base/link7
+	a		http://example.org/link7
 EOF
 
 cmp -s $TMP1 $TMP2
