@@ -1,4 +1,4 @@
-/* C code produced by gperf version 3.0.2 */
+/* C code produced by gperf version 3.0.3 */
 /* Command-line: gperf -a -c -C -o -t -p -k '1,2,$' -D -N lookup_entity unent.hash  */
 
 #if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
@@ -119,6 +119,9 @@ hash (str, len)
 
 #ifdef __GNUC__
 __inline
+#ifdef __GNUC_STDC_INLINE__
+__attribute__ ((__gnu_inline__))
+#endif
 #endif
 const struct _Entity *
 lookup_entity (str, len)
